// LoggingFilter.cs
//
// This library contains an implementation of IHttpModule that logs the
// request and response streams to a log file (specified in web.config)
// using the Filter property of the HttpResponse object associated
// with the incoming request.
//
// The LoggingFilter class implements IHttpModule as needed.  The
// EchoStream class is used by the LogginFilter class to get the job done.
// EchoStream, a derivative of the abstract System.IO.Stream class,
// takes two streams as input to its constructor, and implements the
// Stream abstraction so that all output sent to the EchoStream is
// sent to both of the original streams.  LoggingFilter uses this
// so that the EchoStream can be set as the Filter for the response
// object, causing any output written to the response stream to be
// sent to the original response stream as well as c:\temp\LoggingFilter.log.
//
// This module has been tested with simple ASPX pages as well as simple
// ASMX XML web services.
//
// This filter has a few settings that can be configured (one of which is
// required) that appear in web.config.  Refer to the web.config included
// with this sample for documentation.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Web;
using System.IO;
using System.Configuration;

public class LoggingFilter : IHttpModule
{
    private HttpApplication httpApp;
    private EchoStream      responseStream;
    private string          logFileName;
    private long            maxLogSize;

    public void Init( HttpApplication httpApp )
    {
        this.httpApp = httpApp;

        bool filterEnabled;
        GetSettings(out filterEnabled, out logFileName, out maxLogSize);

        if( !filterEnabled )
            return;

        if( logFileName == null )
            return;

        httpApp.BeginRequest += new EventHandler(OnBeginRequest);
        httpApp.EndRequest += new EventHandler(OnEndRequest);
    }

    void GetSettings( out bool filterEnabled, out string logFileName, out long maxLogSize )
    {
        bool enabledFlagFound = false;

        filterEnabled = false;
        logFileName = null;
        maxLogSize = 0;

        string[] configKeys = ConfigurationSettings.AppSettings.AllKeys;

        foreach( string key in configKeys )
        {
            switch( key )
            {
                case "LoggingFilter.LogFile":
                    logFileName = ConfigurationSettings.AppSettings["LoggingFilter.LogFile"];
                break;

                case "LoggingFilter.Enabled":
                    filterEnabled = (string.Compare(ConfigurationSettings.AppSettings["LoggingFilter.Enabled"], "true", true) == 0);
                    enabledFlagFound = true;
                break;

                case "LoggingFilter.ResetLogSize":
                    maxLogSize = long.Parse(ConfigurationSettings.AppSettings["LoggingFilter.ResetLogSize"]);
                break;
            }
        }

        // If the LoggingFilter.Enabled key was set in the config file,
        // then just leave filterEnabled as it was configured.  If the
        // LoggingFilter.Enabled key wasn't in the config file at all,
        // the default behavior will be to enable the filter as long as
        // a file name was specified.
        //
        if( !enabledFlagFound )
        {
            filterEnabled = (logFileName != null);
        }
    }

    void OnBeginRequest( object sender, EventArgs a )
    {
		bool truncateLog = false;

		if( (maxLogSize > 0) && File.Exists(logFileName) )
		{
			FileInfo fiLogFile = new FileInfo(logFileName);
			truncateLog = (fiLogFile.Length >= maxLogSize);
		}

		// Open the log file (for appending) and log some things
        // about the request that was received.
        //
        Stream responseLog =
            File.Open( logFileName,
                       (truncateLog ? FileMode.Truncate : FileMode.Append),
                       FileAccess.Write );
        LogRequestInfo(responseLog);

        // Set the response filter to refer to the echoing stream bound to
        // the original response stream and the log file.
        //
        responseStream = new EchoStream(httpApp.Response.Filter, responseLog);
        httpApp.Response.Filter = responseStream;
    }

    void OnEndRequest( object sender, EventArgs a )
    {
        if( responseStream != null )
            responseStream.Close();
    }

    void LogRequestInfo( Stream logFile )
    {
        // Make some notes in the log file about the request that was
        // received.
        WriteLine(logFile, "\n\nRequest received: {0}", DateTime.Now);
        WriteLine(logFile, httpApp.Request.Url.ToString());
        WriteLine( logFile, "{0}, type: {1}, length: {2}, encoding: {3}",
                   httpApp.Request.HttpMethod,
                   httpApp.Request.ContentType,
                   httpApp.Request.ContentLength,
                   httpApp.Request.ContentEncoding.EncodingName );
        WriteLine(logFile, "Request follows:");

        // Note where the request stream is currently positioned,
        // make a copy of the request stream to the log file,
        // then restore the request stream position.
        //
        long pos = httpApp.Request.InputStream.Position;
        CopyStream(httpApp.Request.InputStream, logFile);
        httpApp.Request.InputStream.Position = pos;

        WriteLine(logFile, "\nResponse follows:");
    }

    void WriteLine( Stream s, string format, params object[] args )
    {
        string text = string.Format(format + "\n", args);
        byte[] textBytes = System.Text.Encoding.ASCII.GetBytes(text);
        s.Write(textBytes, 0, textBytes.Length);
    }

    void CopyStream( Stream inStream, Stream outStream )
    {
        byte[] buf = new byte[128];
        int bytesRead = 0;

        while( (bytesRead = inStream.Read(buf, 0, buf.Length)) > 0 )
        {
            outStream.Write(buf, 0, bytesRead);
        }
    }

    public void Dispose()
    {
    }
}

public class EchoStream : Stream
{
    private Stream  primaryStream;
    private Stream  otherStream;

    public EchoStream( Stream primaryStream, Stream otherStream )
    {
        this.primaryStream = primaryStream;
        this.otherStream = otherStream;
    }

    public override bool CanRead
    {
        get { return(primaryStream.CanRead); }
    }

    public override bool CanSeek
    {
        get { return(primaryStream.CanSeek); }
    }

    public override bool CanWrite
    {
        get { return(primaryStream.CanWrite); }
    }

    public override long Length
    {
        get { return(primaryStream.Length); }
    }

    public override long Position
    {
        get { return(primaryStream.Position); }
        
        set
        {
            primaryStream.Position = otherStream.Position = value;
        }
    }

    public override long Seek(long offset, SeekOrigin direction)
    {
        long pos = primaryStream.Seek(offset, direction);

        if( otherStream.Seek(offset, direction) != pos )
        {
            throw new ApplicationException("Failed to keep streams in sync while seeking.");
        }

        return(pos);
    }

    public override void SetLength(long length)
    {
        primaryStream.SetLength(length);
        otherStream.SetLength(length);
    }

    public override void Close()
    {
        primaryStream.Close();
        otherStream.Close();
    }

    public override void Flush()
    {
        primaryStream.Flush();
        otherStream.Flush();
    }

    public override int Read( byte[] buffer, int offset, int count )
    {
        int bytesRead = primaryStream.Read(buffer, offset, count);
        otherStream.Position = primaryStream.Position;
        return(bytesRead);
    }

    public override void Write( byte[] buffer, int offset, int count )
    {
        primaryStream.Write(buffer, offset, count);
        otherStream.Write(buffer, offset, count);
    }

    public override int ReadByte()
    {
        int b = primaryStream.ReadByte();
        otherStream.Position = primaryStream.Position;
        return(b);
    }

    public override void WriteByte( byte b )
    {
        primaryStream.WriteByte(b);
        otherStream.WriteByte(b);
    }
}
